#ifndef F_CPU
	#define F_CPU 4000000UL
#endif
#define UART_BAUD 19200

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include "lcd.h"

/* single-bit macros, used for control bits */
#define GLUE(a, b)     a##b
#define SET_(what, p, m) GLUE(what, p) |= (1 << (m))
#define CLR_(what, p, m) GLUE(what, p) &= ~(1 << (m))
#define SET(what, x) SET_(what, x)
#define CLR(what, x) CLR_(what, x)

#define USART_timeout 15	// 65.5ms/LSB
#define power_key D, 2
#define test_pin A, 7
#define lcd_bl B, 3

volatile int8_t timeout_tick=0;

ISR(TIMER0_OVF_vect)	//overflow rate @ 15.26Hz (65.5ms)
{
	timeout_tick++;
}

void timer0_init(void)	//Timer/Counter 0 initialization
{
	TCCR0=0x05;	//Mode 0 (normal, top=0xFF), T0clock = SysClock/1024(=3.906kHz), OC0 output disconnected
	TCNT0=0x00;	// overflow rate = 15.26Hz
	OCR0=0x00;
	TIMSK=0x01; //Timer/Counter Interrupt initialization
}

void uart_init(void)
{
#if F_CPU < 2000000UL && defined(U2X)
  UCSRA = _BV(U2X);             /* improve baud rate error by using 2x clk */
  UBRRL = (F_CPU / (8UL * UART_BAUD)) - 1;
#else
  UBRRL = (F_CPU / (16UL * UART_BAUD)) - 1;
#endif
  UCSRB = _BV(TXEN) | _BV(RXEN); /* tx/rx enable */
}

char USARTReadChar(void)	//This function is used to read the available data from USART.
{
   timeout_tick=0;
   while((!(UCSRA & (1<<RXC))) && (timeout_tick<USART_timeout));	//Wait untill a data is available or timeout
   if(UCSRA & (1<<RXC)) return UDR; else return 0;	//if USART hasn't got data then return 0
}

char USARTReadChar_NTO(void)	//This function is used to read the available data from USART without timeout
{
   while(!(UCSRA & (1<<RXC)));	//Wait untill a data is available
   return UDR;	//if USART has got the data
}

int wait_search_USART(char *search_str)
{
	const uint8_t max_to=20;
	uint8_t to_count=0,index=0;
	char tmp;
	while((index<(strlen(search_str))) && (to_count<max_to))
	{
		tmp=USARTReadChar();
		if(tmp=='\0') to_count++;
		if(tolower(tmp)==search_str[index]) index++; else index=0;
	}
	if(to_count<max_to) return 1; else return 0;
}

void USARTWriteChar(char data)	//This fuction writes the given "data" to the USART which then transmit it via TX line
{
   while(!(UCSRA & (1<<UDRE)));   //Wait untill the transmitter is ready
   UDR=data;   						//Now write the data to USART buffer
}

void send_string(char *to_send) 
   { 
     do 
       { 
         USARTWriteChar(*to_send); 
         to_send++; 
       } 
     while(*to_send!='\0'); //send until null detected
    }

void delay_sec(uint8_t sec)
{
  for (uint8_t j=0; j<sec; j++)
    for (uint8_t i=0; i<10; i++)
      _delay_ms(100);
}

void delay_mnt(uint8_t mnt)
{
  for (uint8_t j=0; j<mnt; j++)
		delay_sec(60);
}

void check_net(void)
{
	while(1)
	{
		send_string("AT+CREG?\r");
		if(wait_search_USART("+creg: 0,1")!=0) break;
	}
}

void sim_powerup(void)
{
	uint8_t wait_count;
	
	CLR(PORT, test_pin);
	SET(PORT, power_key);
	delay_sec(3);
	SET(PORT, test_pin);
	CLR(PORT, power_key);
	delay_sec(2);
	wait_count=0;
	for(uint8_t i=0; i<3; i++) send_string("AT\r");
	while(wait_search_USART("call ready")!=1);
	LCDWriteStringXY(0,1,"SIM300 Turned ON");
	CLR(PORT,lcd_bl);
	send_string("ATE0;Q1\r");		// no command echo and result code
	check_net();
	LCDWriteStringXY(0,1,"Conn to Home Net");
	delay_sec(2);
}

void sysinit(void)
{
	SET(DDR, lcd_bl);		//set LCD backlight control pin to output
	SET(PORT, lcd_bl);
	SET(DDR, power_key);	//set power key control pin to output
	CLR(PORT, power_key);	//default state = low
	uart_init();			//initialize USART
	timer0_init();			//initialize Timer 0
	sei();					//enable global interrupt flag
	InitLCD(LS_BLINK|LS_ULINE);	//Initialize LCD module
	LCDClear();					//Clear the screen
	LCDWriteStringXY(0,0,"  Web Scraping  ");
	LCDWriteStringXY(0,1,"     AppNote    ");
	delay_sec(2);
	SET(DDR, test_pin);
	sim_powerup();
}

void extract_data(char *buf)
{
	uint8_t tmp=0,index;
	
	wait_search_USART("\">");
	while((tmp<0x30)||(tmp>0x39)) tmp=USARTReadChar_NTO();
	index=0;
	do
	{
		buf[index]=tmp;
		tmp=USARTReadChar_NTO();
		index++;
	}	
	while(tmp!='<');
	buf[index]='\0';
}

int main(void)
{
	volatile uint8_t disp_cycle;
	char usd_j[8], usd_b[8], sgd_j[8], sgd_b[8], hkd_j[8], hkd_b[8], chf_j[8], chf_b[8];
		
	sysinit();
	LCDClear();
	LCDWriteStringXY(0,0," Set GPRS Param ");
	send_string("AT+CGDCONT=1,\"IP\",\"telkomsel\"\r");
	send_string("AT+CSTT=\"telkomsel\",\"wap\",\"wap123\"\r");
	send_string("AT+CDNSORIP=0\r");	// switch to ip mode (ip address = 0, domain name = 1)
	delay_sec(2);
	LCDWriteStringXY(0,0,"Start GPRS tasks");
	send_string("AT+CIICR\r");		// brings up wireless connection
	delay_sec(2);
	send_string("AT+CIPSHUT\r");	//start GPRS task = for telkomsel Simpati
	while(wait_search_USART("shut ok")!=1);
	while(1)
	{
		LCDWriteStringXY(0,0,"Connecting to ->");
		LCDWriteStringXY(0,1," www.klikbca.com");
		send_string("AT+CIPSTART=\"TCP\",\"202.6.211.8\",\"80\"\r");	//start up TCP connection to www.klikbca.com
		while(wait_search_USART("connect ok")!=1);
		send_string("AT+CIPSEND\r");	//send data via GPRS
		while(USARTReadChar()!='>');
		LCDWriteStringXY(0,0,"Send request to:");
		send_string("GET / HTTP/1.1\r\n");
		send_string("Host: www.klikbca.com\r\n\r\n");
		USARTWriteChar(26);
		/* Wait data from www.klikbca.com and then extract it */
		wait_search_USART("usd");
		extract_data(usd_j);
		extract_data(usd_b);
		wait_search_USART("sgd");
		extract_data(sgd_j);
		extract_data(sgd_b);
		wait_search_USART("hkd");
		extract_data(hkd_j);
		extract_data(hkd_b);
		wait_search_USART("chf");
		extract_data(chf_j);
		extract_data(chf_b);
		send_string("AT+CIPSHUT\r");
		LCDWriteStringXY(0,1,"Today DD/TT Rate");
		LCDWriteStringXY(0,0," www.klikbca.com");
		delay_sec(5);
		disp_cycle=0;
		while(disp_cycle<60)
		{
			if((disp_cycle & 0x03) == 0x00)
			{
				LCDWriteStringXY(0,0,"J:IDR=");
				LCDWriteString(usd_j);
				LCDWriteString("USD");
				LCDWriteStringXY(0,1,"B:IDR=");
				LCDWriteString(usd_b);
				LCDWriteString("USD");
			} else if((disp_cycle & 0x03) == 0x01)
			{
				LCDWriteStringXY(0,0,"J:IDR=");
				LCDWriteString(sgd_j);
				LCDWriteString("SGD");
				LCDWriteStringXY(0,1,"B:IDR=");
				LCDWriteString(sgd_b);
				LCDWriteString("SGD");
			} else if((disp_cycle & 0x03) == 0x02)
			{
				LCDWriteStringXY(0,0,"J:IDR=");
				LCDWriteString(hkd_j);
				LCDWriteString("HKD");
				LCDWriteStringXY(0,1,"B:IDR=");
				LCDWriteString(hkd_b);
				LCDWriteString("HKD");
			} else if((disp_cycle & 0x03) == 0x03)
			{
				LCDWriteStringXY(0,0,"J:IDR=");
				LCDWriteString(chf_j);
				LCDWriteString("CHF");
				LCDWriteStringXY(0,1,"B:IDR=");
				LCDWriteString(chf_b);
				LCDWriteString("CHF");
			}
			delay_sec(5);
			disp_cycle++;
		}
	}
}